/* Tohle je zdrojovy kod pro BISONa implementujici htmlcheck,
   kontrolor syntaxe HTML dokumentu.

   Syntakticky analyzator

   Jan Kybic a Robert Galiana, 1997

   5.5.97 - pracovni varianta, nutne doplnit	
*/

%{
#include <stdio.h>
#include <string.h>

static int error_count ;  /* zde pocitame pocet chyb */

/* nasledujici dve makra zapnou podrobnou chybovou diagnostiku pri
   syntakticke chybe. Pozdeji nahradime prepsanim pravidel, tak aby rovnou
   detekovala i chyby, priklad viz 'Unknown element'. */

#define YYERROR_VERBOSE 1 
#define YYDEBUG 1

extern int yyleng ;    /* obsahuji kontext, viz. yyerror() nize */
extern char *yytext ;

%}

/* Sekce deklaraci: zde definujeme terminaly */

%token TITLE_BEGIN
%token TITLE_END
%token HTML_BEGIN
%token HTML_END
%token BODY_BEGIN
%token BODY_END
%token HEAD_BEGIN
%token HEAD_END
%token UNKNOWN
%token A_BEGIN
%token A_END
%token ELEM_END
%token PAR_HREF
%token URL
%token WRONG_URL
%token H1_BEGIN
%token H1_END
%token H2_BEGIN
%token H2_END
%token H3_BEGIN
%token H3_END
%token H4_BEGIN
%token H4_END
%token H5_BEGIN
%token H5_END
%token H6_BEGIN
%token H6_END
%token PARA
%token PARA_END
%token TABLE_BEGIN
%token TABLE_END
%token PAR_WIDTH
%token PAR_UNKNOWN
%token PAR_SIZE
%token PAR_LEFT
%token PAR_RIGHT
%token PAR_CENTER
%token PAR_MIDDLE
%token PAR_TOP
%token PAR_BOTTOM
%token PAR_ALIGN
%token PAR_VALIGN
%token I_BEGIN
%token I_END
%token ADDRESS_BEGIN
%token ADDRESS_END
%token HRULE
%token TR_BEGIN
%token TR_END
%token TD_BEGIN
%token TD_END
%token IMG
%token PAR_SRC
%token GRTR
%token BREAK
%token PAR_ALT
%token PAR_NAME
%token LI
%token LI_END
%token UL
%token END_UL
%token SAMP
%token END_SAMP
%token DFN
%token END_DFN
%token STRONG
%token END_STRONG
%token PAR_NAME
%token PAR_NAMET
%token DT_END
%token DD_END
%token DT
%token DL
%token PAR_COMPACT
%token END_DL
%token DD
%token OL
%token END_OL
%token PRE
%token END_PRE
%token CITE 
%token END_CITE
%token PAR_BGCOLOR
%token PAR_TEXT
%token VAR
%token END_VAR
%token B_BEGIN
%token B_END
%token EM_BEGIN
%token EM_END
%token BLOCKQUOTE_BEGIN
%token BLOCKQUOTE_END
%token KBD_BEGIN
%token KBD_END
%token CODE_BEGIN
%token CODE_END
%token EMPHASIS_BEGIN
%token EMPHASIS_END
%token TH_BEGIN
%token TH_END
%token PAR_BORDER
%token PAR_COLSPAN
%token PAR_NOWRAP

%%
/* Sekce gramatickych pravidel. Neterminaly male, terminaly velke */

/* htmldoc je startovaci symbol gramatiky */

htmldoc: HTML_BEGIN head_part body_part html_end ;

html_end: HTML_END 
        | /* empty */ { yyerror("</HTML> expected") ; } 

head_part: HEAD_BEGIN html_text HEAD_END ;

body_part: body_begin html_text body_end ;

body_begin: BODY_BEGIN body_params ELEM_END ;
	 | /* empty */ { yyerror("<BODY> expected.") ; }

body_params: /* empty */ | body_params body_param ;

body_param: par_unknown | bgparm | textparm ;

body_end: BODY_END
         | /* empty */ { yyerror("</BODY> expected.") ; }

html_text: /* empty */
	 | html_text element ;


element:   a_begin html_text A_END  ;
         | TITLE_BEGIN html_text TITLE_END ;
	 | heading
         | I_BEGIN html_text I_END ;
	 | ADDRESS_BEGIN html_text ADDRESS_END ;
	 | table_begin table_text TABLE_END ;
	 | para | PARA_END | image | hrule | BREAK ;
         | UL list_text END_UL ;
	 | OL list_text END_OL ;
	 | dl_begin data_text END_DL ;
	 | GRTR 	{ yyerror("Unexpected symbol >") ; }
	 | STRONG html_text END_STRONG ;
	 | DFN html_text END_DFN ;
	 | SAMP html_text END_SAMP ;
	 | PRE html_text END_PRE ;
	 | CITE html_text END_CITE ;
	 | VAR html_text END_VAR ;
	 | EM_BEGIN html_text EM_END ;
	 | B_BEGIN html_text B_END ;
	 | BLOCKQUOTE_BEGIN html_text BLOCKQUOTE_END ;
	 | KBD_BEGIN html_text KBD_END ;
	 | CODE_BEGIN html_text CODE_END ;
	 | EMPHASIS_BEGIN html_text EMPHASIS_END ;
	 | unknown | error  ;

dl_begin: DL ELEM_END | DL PAR_COMPACT ELEM_END ;

heading:   h1_begin html_text H1_END ;
	 | h2_begin html_text H2_END ;
	 | h3_begin html_text H3_END ;
	 | h4_begin html_text H4_END ;
	 | h5_begin html_text H5_END ;
	 | h6_begin html_text H6_END ;

h1_begin: H1_BEGIN h_params ;
h2_begin: H2_BEGIN h_params ;
h3_begin: H3_BEGIN h_params ;
h4_begin: H4_BEGIN h_params ;
h5_begin: H5_BEGIN h_params ;
h6_begin: H6_BEGIN h_params ;
          
h_params: h_prms ELEM_END ;
h_prms: /* empty */ | h_prms h_prm ;

h_prm: par_unknown | align | valign ;


bgparm: PAR_BGCOLOR PAR_NAME ;

textparm: PAR_TEXT PAR_NAME ;


list_text: /* empty */ | element list_text_sequence 
           | list_text_sequence list_text ;

list_text_sequence: LI list_text_inner LI_END  ;

list_text_inner: /* empty */ | element list_text_inner ;

data_text: /* empty */ | element data_text_sequence 
         | data_text_sequence data_text ;

data_text_sequence: DD data_text_inner DD_END 
                  | DT data_text_inner DT_END ;

data_text_inner: /* empty */ | element data_text_inner ;           

unknown: UNKNOWN { yyerror("Unknown element.") ; } 

hrule: HRULE hrule_params ELEM_END ;

hrule_params: /* empty */
	     | hrule_params hrule_param ;

hrule_param: PAR_SIZE ;
           | par_unknown ;

par_unknown: PAR_UNKNOWN | PAR_NAME { yyerror("Unknown parameter.");}

para:	PARA para_params ELEM_END ;

para_params: /* empty */ | align | valign | par_unknown ;

table_begin: TABLE_BEGIN table_params ELEM_END ;

table_params: /* empty */
	     | table_params table_param ;

table_param: PAR_COLSPAN | PAR_BORDER | PAR_WIDTH | par_unknown ;

table_text: /* empty */
           | tr_begin table_row TR_END table_text ;
           | tr_begin table_row table_text ;

table_row: /* empty */
	  | table_row td_begin html_text td_end ;
	  | table_row th_begin html_text TH_END ;
	  | error ;

td_end: TD_END | /* empty */ ;

th_begin: TH_BEGIN trtd_params ELEM_END ;

tr_begin: TR_BEGIN trtd_params ELEM_END ;

trtd_params: /* empty */
	  | trtd_params trtd_param ;

trtd_param: align | valign | PAR_COLSPAN | PAR_WIDTH | PAR_NOWRAP
           | par_unknown ;

align: PAR_ALIGN align_pos ;

valign: PAR_VALIGN align_pos ;

align_pos: PAR_LEFT | PAR_RIGHT | PAR_CENTER 
         | PAR_MIDDLE | PAR_BOTTOM | PAR_TOP | par_unknown ;

td_begin: TD_BEGIN trtd_params ELEM_END ;

a_begin: A_BEGIN a_params ELEM_END ;

wrong_url: WRONG_URL { yyerror("Malformed URL.") ; }

url:	URL | wrong_url ;
     
a_params: PAR_HREF url | par_name | par_unknown ;

par_name: PAR_NAMET PAR_NAME ;

image: IMG img_params ELEM_END ;

img_params: img_param | img_params img_param ;

img_param: PAR_SRC url | align | valign | par_unknown | alt ;

alt:	PAR_ALT PAR_NAME ;


%%
/* Sekce C kodu */


extern int lineno ; /* definovana v htmlcheck.yy */

#define CLEN 10 /* delka kontextu +1 */

int yyerror(char *s)
{ 
  char buf[CLEN] ;
  
  if (yyleng==0) strncpy(buf,"EOF",CLEN) ;
            else { strncpy(buf,yytext,CLEN) ;
                   buf[CLEN-1]='\0' ;
                 }

  printf("Line: %d, read '%s': %s\n",lineno,buf,s) ;
  error_count++ ;
  return 0 ;
}


/* Hlavni program */
int main()
{ int result ;
  
  puts("htmlcheck - HTML syntax checker, (c) Kybic, Galiana, 1997") ;
  puts("Reading from standard input...") ;
  
  error_count=0 ; lineno=1 ;
  yyparse() ;
  
  if (error_count==0) puts("\nhtmlcheck finished, document O.K. Bye!") ;
                 else printf("\nhtmlcheck finished, %d errors found. Bye!\n",
                    error_count) ;

  return result ;
}











