/* Tohle je zdrojovy kod pro FLEX implementujici htmlcheck,
   kontrolor syntaxe HTML dokumentu.

   Lexikalni analyzator.

   Jan Kybic a Robert Galiana, 1997

   1.4.1997
*/

/* Tohle je jen pracovni varianta, nutne dopracovani */


/* Deklarace maker */

%{
#include "htmlcheck.tab.h"     /* tohle vytvori bison */
int lineno ;                   /* cislo radky */
static int follow_state ;
#if 0
/* ladici varianta R  */
#define R(x) { printf("(%d=%s)",x,yytext); return x ; }
#else

/* pracovni varianta R */
#define R(x) return x ;
#endif

#define INCR { lineno++ ; }
%}
	/* nasleduji deklarace pozdeji pouzitelnych zkratek */

DIGIT	[0-9]
LETTER	[a-zA-Z]
ALPHA	({DIGIT}|{LETTER}|"-"|"_")
ALPHAD  ({ALPHA}|".") 
ALPHAT  ({ALPHA}|~)
PRINTABLE [^\n"]
SPACE   [ \t\r]
SPACENL ({SPACE}|\n)
DELIM   [^a-zA-Z]
RESOURCE ((http|HTTP|ftp|FTP)"://")
HOSTNAME {ALPHAD}*
PATHNAME (\/)?(({ALPHAT}{ALPHAD}*)|({ALPHAD}+))(\/{ALPHAD}+)*
FILENAME (\/)?({ALPHAD}*)?
MAILURL ("mailto:"|"MAILTO:"){ALPHAD}+("@"{ALPHAD}+)?  
LABEL	"#"{ALPHA}+    
PORT	":"{DIGIT}+
URL	 ({RESOURCE}?{HOSTNAME}?{PORT}?{PATHNAME}?{FILENAME}{LABEL}?)|{MAILURL}
EQS	{SPACE}*"="{SPACE}*

/* stavy - popis viz. dale*/
%x	S1 S2 S3 S4 S5 S6 S7 S8

%%
		
	/* vypoustime znaky mezi elementy */

[^<\n>]*	/* ignoruj */
\n		INCR ; /* pocitej radky */
">"		R(GRTR) ;

	/* ruzne jednoduche elementy (bez parametru) */
	/* musime je zadat malymi i velkymi, mixy nepovolime */

"<HTML>"|"<html>"	R(HTML_BEGIN) ;
"</HTML>"|"</html>"	R(HTML_END) ;
"<BODY"|"<body"		{ BEGIN(S3) ; R(BODY_BEGIN) ; }
"</BODY>"|"</body>"	R(BODY_END) ;
"<TITLE>"|"<title>"	R(TITLE_BEGIN) ;
"</TITLE>"|"</title>"	R(TITLE_END) ;
"<HEAD>"|"<head>"	R(HEAD_BEGIN) ;
"</HEAD>"|"</head>"	R(HEAD_END) ;
"</A>"|"</a>"		R(A_END) ;

"<H1"|"<h1"		{ BEGIN(S3) ; R(H1_BEGIN) ; }
"</H1>"|"</h1>"		R(H1_END) ;
"<H2"|"<h2"		{ BEGIN(S3) ; R(H2_BEGIN) ; }
"</H2>"|"</h2>"		R(H2_END) ;
"<H3"|"<h3"		{ BEGIN(S3) ; R(H3_BEGIN) ; }
"</H3>"|"</h3>"		R(H3_END) ;
"<H4"|"<h4"		{ BEGIN(S3) ; R(H4_BEGIN) ; }
"</H4>"|"</h4>"		R(H4_END) ;
"<H5"|"<h5"		{ BEGIN(S3) ; R(H5_BEGIN) ; }
"</H5>"|"</h5>"		R(H5_END) ;
"<H6"|"<h6"		{ BEGIN(S3) ; R(H6_BEGIN) ; }
"</H6>"|"</h6>"		R(H6_END) ;

"<I>"|"<i>"		R(I_BEGIN) ;
"</I>"|"</i>"		R(I_END) ;
"<B>"|"<b>"		R(B_BEGIN) ;
"</B>"|"</b>"		R(B_END) ;
"<EM>"|"<em>"		R(EM_BEGIN) ;
"</EM>"|"</em>"		R(EM_END) ;
"<BLOCKQUOTE>"|"<blockquote>"		R(BLOCKQUOTE_BEGIN) ;
"</BLOCKQUOTE>"|"</blockquote>"		R(BLOCKQUOTE_END) ;
"<KBD>"|"<kbd>"		R(KBD_BEGIN) ;
"</KBD>"|"</kbd>"	R(KBD_END) ;
"<CODE>"|"<code>"	R(CODE_BEGIN) ;
"</CODE>"|"</code>"	R(CODE_END) ;
"<EMPHASIS>"|"<emphasis>" R(EMPHASIS_BEGIN) ;
"</EMPHASIS>"|"</emphasis>" R(EMPHASIS_END) ;



"<P"|"<p"		{ BEGIN(S3) ; R(PARA) ;}
"</P>"|"</p>"		R(PARA_END) ;
"<BR>"|"<br>"		R(BREAK) ;
"<LI>"|"<li>"		R(LI) ;
"</LI>"|"</li>"		R(LI_END) ;
"</DD>"|"</dd>"		R(DD_END) ;
"</DT>"|"</dt>"		R(DT_END) ;
"<UL>"|"<ul>"		R(UL) ;
"</UL>"|"</ul>"		R(END_UL) ;
"<DFN>"|"<dfn>"		R(DFN) ;
"</DFN>"|"</dfn>"	R(END_DFN) ;
"<SAMP>"|"<samp>"	R(SAMP) ;
"</SAMP>"|"</samp>"	R(END_SAMP) ;
"<STRONG>"|"<strong>"   R(STRONG) ;
"</STRONG>"|"</strong>" R(END_STRONG) ;
"<DL"|"<dl"		{ BEGIN(S3) ; R(DL) ; }
"</DL>"|"</dl>"		R(END_DL) ;
"<DT>"|"<dt>"		R(DT) ;
"<DD>"|"<dd>"		R(DD) ;
"<OL>"|"<ol>"		R(OL) ;
"</OL>"|"</ol>"		R(END_OL) ;
"<PRE>"|"<pre>"		R(PRE) ;
"</PRE>"|"</pre>"	R(END_PRE) ;
"<CITE>"|"<cite>"	R(CITE) ;
"</CITE>"|"</cite>"	R(END_CITE) ;
"<VAR>"|"<var>"		R(VAR) ;
"</VAR>"|"</var>"	R(END_VAR) ;
"<TH"|"<th"		{ BEGIN(S3) ; R(TH_BEGIN) ;}
"</TH>"|"</th>"		R(TH_END) ;


("<TABLE"|"<table")/{DELIM}	{ BEGIN(S3) ; 
				  R(TABLE_BEGIN) ;	}
"</TABLE>"|"</table>"		R(TABLE_END) ;
("<TR"|"<tr")/{DELIM}		{ BEGIN(S3) ; 
				  R(TR_BEGIN) ;	}
"</TR>"|"</tr>"			  R(TR_END) ;
("<TD"|"<td"|"<Td")/{DELIM}	{ BEGIN(S3) ; 
				  R(TD_BEGIN) ;	}
"</TD>"|"</td>"|"</Td>"		R(TD_END) ;
"<HR"|"<hr"			{ BEGIN(S3) ; R(HRULE) ; }

("<IMG"|"<img")/{DELIM}		{ BEGIN(S3) ; 
				  R(IMG) ;	}


"<ADDRESS>"|"<address>"		R(ADDRESS_BEGIN) ;
"</ADDRESS>"|"</address>"	R(ADDRESS_END) ;

("<A"|"<a")/{DELIM}	{ /* zacatek elementu s parametry */
			BEGIN(S3) ; 
			R(A_BEGIN) ;
			}

"<!--"		{ /* zacatek komentare */
		  BEGIN(S1) ; /* stav preskakovani komentaru */
		}

"<"		{ /* zacatek neznameho elementu */
		BEGIN(S2) ;
		}

	/* stav S1 zpracovava komentare */
	/* preskoc komentare, pritom pocitej radky */

<S1>\n		|
<S1>-\n		|
<S1>--\n	INCR ;  
<S1>[^-\n]*	|
<S1>-[^-\n]	|
<S1>--[^>\n]	/* ignoruj */
<S1>"-->"	{ konec_komentare: ;
		  BEGIN(0) ; /* konec komentare */
		}
	

	/* stav S2 zpracovava neznamy element */

<S2>[^>\n]*	R(UNKNOWN) ; /* ignoruj vnitrek neznameho elementu */
<S2>\n		INCR ;
<S2>">"		BEGIN(0) ;


		/* stav S3 zpracovava element s parametry */

<S3>\n			INCR ;
<S3>{SPACE}		/* ignoruj */
<S3>(HREF|href)		{ follow_state=S4 ; BEGIN(S6) ; /* zacatek parametru url */                      R(PAR_HREF) ;  
			}
<S3>(width|WIDTH)  	{ follow_state=S8 ; BEGIN(S6) ; R(PAR_WIDTH) ; }
<S3>(align|ALIGN)	{ follow_state=S3 ; BEGIN(S6) ; R(PAR_ALIGN) ; }
<S3>(valign|VALIGN)     { follow_state=S3 ; BEGIN(S6) ; R(PAR_VALIGN) ; }
<S3>(left|LEFT)		R(PAR_LEFT) ;
<S3>(right|RIGHT)	R(PAR_RIGHT) ;
<S3>(center|CENTER)	R(PAR_CENTER) ;
<S3>(size|SIZE)         { follow_state=S8 ; BEGIN(S6) ; R(PAR_SIZE) ; }
<S3>(border|BORDER)     { follow_state=S8 ; BEGIN(S6) ; R(PAR_BORDER) ; }
<S3>(colspan|COLSPAN)   { follow_state=S8 ; BEGIN(S6) ; R(PAR_COLSPAN) ; }
<S3>(top|TOP)		R(PAR_TOP) ;
<S3>(middle|MIDDLE)	R(PAR_MIDDLE) ;
<S3>(bottom|BOTTOM)	R(PAR_BOTTOM) ;
<S3>(src|SRC)		{ follow_state=S4 ; BEGIN(S6) ; R(PAR_SRC) ; }
<S3>(alt|ALT)		{ follow_state=S3 ; BEGIN(S6) ; R(PAR_ALT) ; }
<S3>(name|NAME|Name)    { follow_state=S3 ; BEGIN(S6) ; R(PAR_NAMET) ; }
<S3>(compact|COMPACT)	R(PAR_COMPACT) ;
<S3>(bgcolor|BGCOLOR)   { follow_state=S3 ; BEGIN(S6) ;  R(PAR_BGCOLOR) ;}
<S3>(text|TEXT)         { follow_state=S3 ; BEGIN(S6) ;  R(PAR_TEXT) ;}
<S3>(nowrap|NOWRAP) 	R(PAR_NOWRAP) ;
<S3>\"{PRINTABLE}*\" |
<S3>("#"?{ALPHAD})+         R(PAR_NAME) ;
<S3>">"			{ BEGIN(0) ; R(ELEM_END) ;/* konec elementu */}
<S3>.			{ BEGIN(S5) ; R(PAR_UNKNOWN) ; } /* neznamy parametr */
	


		/* S4 zpracovava URL */

<S4>\"{URL}\"   |
<S4>{URL}/({SPACE}|\n|\>)  { BEGIN(S3) ; R(URL) ; }
<S4>\"[^"]*\"    |
<S4>[^ \"\t\n\>]*           { BEGIN(S3) ; R(WRONG_URL) ; }

	/* stav S5 zpracovava neznamy parametr */

<S5>[^>\n ]*	/* ignoruj vnitrek neznameho elementu */
<S5>{SPACENL}|">" { BEGIN(S3) ; yyless(0) ; }

	/* stav S6 preskoci rovnitka a prejde do stavu follow_state */

<S6>{SPACE}*
<S6>\n		INCR ;
<S6>"="		BEGIN(S7) ;
<S6>.		{ BEGIN(follow_state) ; yyless(0) ; }

<S7>{SPACE}*
<S7>\n		INCR ;
<S7>.		{ BEGIN(follow_state) ; yyless(0) ; }

<S8>[^{SPACENL}">"]* BEGIN(S3) ;
<S8>.		{ BEGIN(S3) ; yyless(0) ; }

%%

/* jen jeden vstupni soubor */
int yywrap() { return 1 ; }















